/*
 *按照教程学习串口的编程
 *网址1：http://blog.csdn.net/shanzhizi/article/details/9241393
 *网址2：http://blog.csdn.net/shui1025701856/article/details/7571686
 *网址3：http://wenku.baidu.com/link?url=TsZc-rSDq_8ayNdWXYzkJRHLEowtx7OQMiFj_BH4M3VfvDrw4hU785aIciwS1xAmViJmz6pN0ety1_Zv8-AM2OAflOB3-UiMu9RvcSQKrAG
 */
//头文件
#include <stdio.h> 	//标准输入输出定义
#include <stdlib.h>	//标准函数库定义
#include <unistd.h>	//Unix标准函数定义
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 	//文件控制定义
#include <termios.h>	//POSIX中断控制定义
#include <errno.h>	//错误号定义
#include <string.h>


//定义波特率数组
int speed_arr[] = {B115200, B38400, B19200, B9600, B4800, B2400, B1200, B300};
int name_arr[] = {115200,38400, 19200, 9600, 4800, 2400, 1200, 300};

//打开串口
int open_port(void)
{
	int fd;		//串口的标识符
	//O_NOCTTY用来告诉Linux这个程序不会成为“控制终端”
	//O_NDELAY用来告诉Linux这个程序不关心DCD信号
	fd=open("/dev/ttyS1",O_RDWR | O_NOCTTY | O_NDELAY);
	if(fd == -1)
	{
		//不能打开串口
		perror("open_port: Unable to open /dev/ttyS0 -");
		return(fd);
	}
	else
	{
		fcntl(fd, F_SETFL, 0);
		printf("open ttys1 .....\n");
		return(fd);
	}
}

//设置波特率
void set_speed_and_parity(int fd, int speed)
{
	int i=0;		//设置循环标志——注意不要在for内设置，否则会出错
	struct termios Opt;	//定义termios结构
	if(tcgetattr(fd,&Opt)!=0)
	{
		perror("tcgetattr fd");
		return;
	}
	for(i = 0; i < 8 ; i++)
    	{
        	if(speed == name_arr[i])
        	{	
            		tcflush(fd, TCIOFLUSH);
            		cfsetispeed(&Opt, speed_arr[i]);
            		cfsetospeed(&Opt, speed_arr[i]);
			/*tcsetattr函数标志：
			TCSANOW：立即执行而不等待数据发送或者接受完成。
			TCSADRAIN：等待所有数据传递完成后执行。
			TCSAFLUSH：Flush input and output buffers and make the change
			*/
            		if(tcsetattr(fd, TCSANOW, &Opt) != 0)
            		{	
                		perror("tcsetattr fd");
                		return;
            		}
            		tcflush(fd, TCIOFLUSH);
        	}
    	}
	//设置奇偶校验——默认8个数据位、没有校验位
        
    Opt.c_cflag &= ~PARENB;
    Opt.c_cflag &= ~CSTOPB;
    Opt.c_cflag &= ~CSIZE;
    Opt.c_cflag |= CS8;

    Opt.c_cflag &= ~INPCK;
    Opt.c_cflag |= (CLOCAL | CREAD);
 
    Opt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
 
    Opt.c_oflag &= ~OPOST;
    Opt.c_oflag &= ~(ONLCR | OCRNL);    
 
    Opt.c_iflag &= ~(ICRNL | INLCR);
    Opt.c_iflag &= ~(IXON | IXOFF | IXANY);    
    
    Opt.c_cc[VTIME] = 0;
    Opt.c_cc[VMIN] = 1;
    
    tcflush(fd, TCIOFLUSH);

}


/**
  *串口发送数据函数
  *fd：串口描述符
  *data：待发送数据
  *datalen：数据长度
  */
int serial_write(int fd ,char *data, int datalen)
{
	int len=0;
	//获取实际传输数据的长度
	len=write(fd,data,datalen);
	printf("send data OK! datalen=%d\n",len);
	/*if(len==datalen)
	{
		return len;
	}
	else
	{
		//TCOFLUSH刷新写入的数据，但是不会传送
		tcflush(fd,TCOFLUSH);
		return -1;
	}*/
	return len;	
}

/** 
  *串口接收数据 
  *要求启动后，在pc端发送ascii文件 
  */ 
int serial_read(int fd,char buff[],int datalen)
{
	/*char *readbuf;
	int readf=0,i=0,seriallen=0;
	readbuf=buff;//预先保存数据
	printf("ready for receiving data...\n");
	readf=read(fd,readbuf,1);
	if(readf==-1)
	{
		perror("read bad1");//读取状态判断
	}
	printf("receive 1 data");
	//正式开始读取数据
	while(*readbuf!='\n')
	{
		seriallen++;
		readbuf++;
		readf=read(fd,readbuf,1);
		if(readf==-1)
		{
			perror("read bad2");
		}
		printf("seriallen=%d\n",seriallen);
	}
	datalen=seriallen;
	//将接收到的数据提取出来
	printf("The data received is:\n");
	for(i=0;i<seriallen;i++)
	{
		printf("%c",buff[i]);
	}
	printf("\n");
	return 0;
	*/
	int nread=0;
	printf("Ready for receiving data...");
	nread=read(fd,buff,datalen);
	if(nread>0)
	{
		printf("readlength=%d\n",nread);
		buff[nread]='\0';
		printf("%s\n",buff);
		sleep(3);
	}
	return nread;
}

int main(void)
{
	int fd;
	int nread,i,n =0, len = 0;
	char testbuff[]="Hello\r\n";
	char readbuff[256];
        char writebuff[256];
	
	//打开串口
	if((fd=open_port())<0)
    	{
        	perror("open_port error");
        	return 0;
    	}
	//设置波特率和校验位
	set_speed_and_parity(fd,115200);
	
	printf("fd=%d\n",fd);
	
	//尝试先写内容
#if 0	
	nread=write(fd,testbuff,8);
	if(nread==-1)
	{
		printf("write bad");
	}
	else{
		printf("the test is ok!");
	}
#endif	
	//循环读取串口并输出
#if 0	
	while(1)
	{
		printf("enter the while loop\n");
		bzero(readbuf,sizeof(readbuf));
		datalen=serial_read(fd,readbuf,sizefo(readbuf));
		if(datalen > 0)
		{
	        if(readbuf[0]=='O'&&buff[1]=='F'&&buff[2]=='F')
		    {
			    break;
		    }
		    serial_write(fd,buff,datalen);
		}

		sleep(2)	
	
	}
#endif
while(1)
    {    
    	printf("enter the while 1 loop\n");
        bzero(readbuff, sizeof(readbuff)); 
        bzero(writebuff, sizeof(writebuff));
 
        while( (n = read(fd, readbuff, sizeof(readbuff))) > 0 )
        {
            printf("enter the while 2 loop\n");
            printf("readbuf:%s\n",readbuff);
#if 0
            for(i = len; i < (len + n); i++)
            {
                writebuff[i] = readbuff[i - len];
            }
            len += n;
#endif
            writebuff[0] = readbuff[0];
        }
        writebuff[1] = '\0';
              
        printf("Len %d \n", len);
        printf("%s \n", writebuff);
 
        n = write(fd, writebuff, len);
        printf("write %d chars\n",n);
        
        sleep(2);	
    }
	//关闭串口
	close(fd);
	return 0;
}




